{
  description = "HELPS";

  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs";
    flake-utils.url = "github:numtide/flake-utils";
  };

  outputs = { self, nixpkgs, flake-utils }:
    flake-utils.lib.eachDefaultSystem (system: let
      pkgs              = nixpkgs.legacyPackages.${system};
      packageName       = "HELPS";
      containerPackageName = "${packageName}-container";

      haskellPackages = pkgs.haskellPackages;

      enableSharedExecutables = false;
      enableSharedLibraries = false;

      HELPS = pkgs.haskell.lib.justStaticExecutables self.packages.${system}.default;
    in {
      packages.${packageName} =
        haskellPackages.callCabal2nix packageName self rec {};

      packages.default = self.packages.${system}.${packageName};
      defaultPackage = self.packages.${system}.default;

      devShells.default = pkgs.mkShell {
        buildInputs = with pkgs; [
          ghcid
          cabal-install
          ghc
        ];
        inputsFrom = builtins.attrValues self.packages.${system};
      };
      devShell = self.devShells.${system}.default;

      packages.${containerPackageName} = pkgs.dockerTools.buildImage {
        name = "HELPS";

        copyToRoot = pkgs.buildEnv {
          name = "image-root";
          paths = [ HELPS ];
          pathsToLink = [ "/bin" ];
        };
        tag = "latest";
        config = {
          Cmd = [
            "/bin/Purr"
          ];
          WorkingDir = "/app";
          ExposedPorts = {
            "3000/tcp" = {};
          };
          extraCommands = ''
          '';
        };
      };
    });
}
