module Logger where

import Core
import Database

import Data.Time                  (getCurrentTime, UTCTime)
import Effectful
import Effectful.Dispatch.Dynamic
import Effectful.Error.Static     (Error, throwError)
import Effectful.Reader.Static
import GHC.Generics                (Generic)
import Servant                     hiding ((:>))

type LogToDatabase es = (Database :> es, Error ServerError :> es, IOE :> es)

type instance DispatchOf Logger = 'Dynamic

writeLog :: (Logger :> es, Error ServerError :> es) 
         => LogLevel -> String -> Eff es ()
writeLog level msg = send (WriteLog level msg)

runLoggerConsole :: (Error ServerError :> es, IOE :> es) => 
  Eff (Logger : es) a -> Eff es a
runLoggerConsole = interpret $ \_ -> \case
  WriteLog level msg -> do
    time <- liftIOE getCurrentTime
    liftIOE $ putStrLn 
       $ "TIMESTAMP: " ++ show time  ++ "\n"
      ++ "LEVEL: "     ++ show level ++ "\n"
      ++ "MESSAGE: "   ++ msg

runLoggerPSQL :: LogToDatabase es => Eff (Logger : es) a -> Eff es a
runLoggerPSQL = interpret $ \_ -> \case
  WriteLog level msg -> 
    databaseWrite 
      ("INSERT INTO logs (level, message) VALUES (?,?);", (level, msg))
      
{-
CREATE TABLE logs (
  id SERIAL PRIMARY KEY,
  level VARCHAR(10) NOT NULL, 
  message TEXT NOT NULL,
  timestamp TIMESTAMPTZ NOT NULL DEFAULT NOW(),
  source VARCHAR(100),
  context JSONB
);
-}
